package com.jboss.datagrid.test.notifications;

import com.jboss.datagrid.test.notifications.listeners.DistributionListener;
import com.jboss.datagrid.test.notifications.utils.DeploymentBuilder;
import com.jboss.datagrid.test.notifications.utils.ITestUtils;
import org.infinispan.manager.DefaultCacheManager;
import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.container.test.api.OperateOnDeployment;
import org.jboss.arquillian.container.test.api.TargetsContainer;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.arquillian.junit.InSequence;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.junit.Test;
import org.junit.runner.RunWith;

import static com.jboss.datagrid.test.notifications.utils.ITestUtils.Condition;
import static com.jboss.datagrid.test.notifications.utils.ITestUtils.eventually;
import static junit.framework.Assert.assertTrue;

/**
 * Tests verifying that in the Distribution clustering mode, the corresponding events are fired and notifications
 * are asynchronous.
 *
 * @author Anna Manukyan
 */
@RunWith(Arquillian.class)
public class DistributionNotificationIT {
   private static DefaultCacheManager dfc1 = null;
   private static DefaultCacheManager dfc2 = null;

   private static String cacheName = "testCache";
   private String key = "key";
   private String value = "value";
   private static DistributionListener listenerNode1;
   private static DistributionListener listenerNode2;
   private static int testCounter = 0;

   /**
    * Creates a deployment on the first node.
    * @return  the war file for the first node.
    */
   @Deployment(name = "node1")
   @TargetsContainer("container1")
   public static WebArchive createDeployment() {
      return DeploymentBuilder.createTestsWebArchive(DeploymentBuilder.WAR_NAME);
   }

   /**
    * Creates deployment for the second node.
    * @return     the war for the second node.
    */
   @Deployment(name = "node2")
   @TargetsContainer("container2")
   public static WebArchive createDeployment2() {
      return DeploymentBuilder.createTestsWebArchive(DeploymentBuilder.WAR_NAME);
   }

   /**
    * Creates and starts Distributed cache on node1.
    * @throws Exception
    */
   @Test
   @OperateOnDeployment("node1")
   @InSequence(1)
   public void testDataRehashEventNode1() throws Exception {
      System.out.println("The cache is starting on the node1.");
      dfc1 = ITestUtils.configureCacheManagerForCluster(cacheName, false);

      dfc1.getCache(cacheName).clear();
      listenerNode1 = new DistributionListener("node1");
      dfc1.getCache(cacheName).addListener(listenerNode1);
   }

   /**
    * Creates and starts Distributed cache on node2. Puts data onto the cache.
    * @throws Exception
    */
   @Test
   @OperateOnDeployment("node2")
   @InSequence(2)
   public void testDataRehashEventNode2() throws Exception {
      System.out.println("The cache is starting on the node2.");

      dfc2 = ITestUtils.configureCacheManagerForCluster(cacheName, false);
      dfc2.getCache(cacheName).clear();

      listenerNode2 = new DistributionListener("node2");
      dfc2.getCache(cacheName).addListener(listenerNode2);

      for(int i = 0; i < 100; i++) {
         dfc2.getCache(cacheName).put(key + i, value + i);
      }
   }

   /**
    * Verifying that the data has been rehashed on node1, the event DataRehashed was fired and the notification was
    * asynchronous.
    * @throws Exception
    */
   @Test
   @OperateOnDeployment("node1")
   @InSequence(3)
   public void testDataRehashEventNode1Verifying() throws Exception {

      eventually("The rehash should happen.", new Condition() {
         @Override
         public boolean isSatisfied() throws Exception {
            return listenerNode1.isRehashed;
         }
      }, 15000, 15);

      assertTrue("The rehash listener thread should be different rather than the current.", Thread.currentThread() != listenerNode1.threadRunningNotif);

      assertTrue("The topology should be changed so far on node1.", listenerNode1.isTopologyChanged);
      assertTrue("The topology should be changed with different thread rather than this one on node1.", listenerNode1.topologyThreadRunningNotif != Thread.currentThread());
   }

   @Test
   @OperateOnDeployment("node1")
   @InSequence(5)
   public void stopCacheOnNode1() {
      System.out.println("Stopping cache on node1");
      dfc1.removeListener(listenerNode1);
       ITestUtils.shutDownCache(dfc1, cacheName);
   }

   @Test
   @OperateOnDeployment("node2")
   @InSequence(6)
   public void stopCacheOnNode2() {
      System.out.println("Stopping cache on node2");
      dfc2.removeListener(listenerNode2);
       ITestUtils.shutDownCache(dfc2, cacheName);
   }
}