package com.jboss.datagrid.test.notifications;

import com.jboss.datagrid.test.notifications.listeners.InvalidationListener;
import com.jboss.datagrid.test.notifications.utils.DeploymentBuilder;
import com.jboss.datagrid.test.notifications.utils.ITestUtils;
import org.infinispan.manager.DefaultCacheManager;
import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.container.test.api.OperateOnDeployment;
import org.jboss.arquillian.container.test.api.TargetsContainer;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.arquillian.junit.InSequence;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.junit.Test;
import org.junit.runner.RunWith;

import static com.jboss.datagrid.test.notifications.utils.ITestUtils.Condition;
import static com.jboss.datagrid.test.notifications.utils.ITestUtils.eventually;
import static junit.framework.Assert.assertTrue;
import static org.junit.Assert.assertEquals;

/**
 * Tests covering asynchronous notifications fired during actions related to infinispan cache.
 *
 * @author Anna Manukyan
 */
@RunWith(Arquillian.class)
public class InvalidationNotificationIT {
   private static DefaultCacheManager dfc1 = null;
   private static DefaultCacheManager dfc2 = null;

   private static String cacheName = "testCache";
   private String key = "key";
   private String value = "value";

   private static InvalidationListener listenerNode1;
   private static InvalidationListener listenerNode2;

   /**
    * Creates deployment on the first node.
    * @return     the war file created for the first node.
    */
   @Deployment(name = "node1")
   @TargetsContainer("container1")
   public static WebArchive createDeployment() {
      return DeploymentBuilder.createTestsWebArchive(DeploymentBuilder.WAR_NAME);
   }

   /**
    * Creates deployment on the second node.
    * @return     the war file created for the second node.
    */
   @Deployment(name = "node2")
   @TargetsContainer("container2")
   public static WebArchive createDeployment2() {
      return DeploymentBuilder.createTestsWebArchive(DeploymentBuilder.WAR_NAME);
   }

   /**
    * The JDG configuration is clustered - Invalidation mode. The purpose of the test is to verify that the
    * Invalidation event is asynchronous. With this action the data is pushed to the cache.
    * @throws Exception
    */
   @Test
   @OperateOnDeployment("node1")
   @InSequence(1)
   public void testKeyCRUD() throws Exception {
      dfc1 = ITestUtils.configureCacheManagerForCluster(cacheName, true);

      listenerNode1 = new InvalidationListener("node1");
      dfc1.getCache(cacheName).addListener(listenerNode1);

      for(int i = 0; i < 100; i++) {
         dfc1.getCache(cacheName).put(key + i, value + i);
      }

      assertTrue("The counter in listener class should be lower or equal than the real one, but is " + listenerNode1.keyId, listenerNode1.keyId.get() <= 100);
      assertTrue("The thread should be not the same running this part.", Thread.currentThread() != listenerNode1.threadRunningNotif);

      listenerNode1.keyId.set(0);
   }

   /**
    * The test verifies that the elements put in node1 are not replicated (as the cache is in invalidation mode). Then
    * the same keys are put to the cache on the second node with different values. The cache on node1 should be invalidated.
    * @throws Exception
    */
   @Test
   @OperateOnDeployment("node2")
   @InSequence(2)
   public void testKeyCRUD1() throws Exception {
      listenerNode2 = new InvalidationListener("node2");
      dfc2 = ITestUtils.configureCacheManagerForCluster(cacheName, true);
      dfc2.getCache(cacheName).addListener(listenerNode2);

      assertEquals("The number of elements in the cache should be 0", 0, dfc2.getCache(cacheName).size());

      for(int i = 0; i < 50; i++) {
         dfc2.getCache(cacheName).put(key + i, "otherValue");
      }

      assertTrue("The counter in listener class should be lower than or equal to the real one, but is " + listenerNode2.keyId, listenerNode2.keyId.get() <= 50);
      assertTrue("The current thread should not be the same as the one handling events.", Thread.currentThread() != listenerNode2.threadRunningNotif);
   }

   /**
    * Checks that the cache on node1 is invalidated and the CacheInvalidated event is fired and the notification went
    * asynchronously.
    *
    * @throws Exception
    */
   @Test
   @OperateOnDeployment("node1")
   @InSequence(3)
   public void testNode1Invalidation() throws Exception {

      eventually("The elements should be invalidated.", new Condition() {
         @Override
         public boolean isSatisfied() throws Exception {
            return listenerNode1.isInvalidated;
         }
      }, 15000, 15);

      assertTrue("The invalidation should be done asynchronously", listenerNode1.threadRunningNotif != Thread.currentThread());
      assertTrue("The number of called invalidation method should be <= 50, but is " + listenerNode1.keyId.get(), listenerNode1.keyId.get() <= 50);
   }

   @Test
   @OperateOnDeployment("node2")
   @InSequence(4)
   public void stopCacheOnNode2() {
      System.out.println("Shutting down cachemanager on node2.");
      dfc2.removeListener(listenerNode2);
      ITestUtils.shutDownCache(dfc2, cacheName);
   }

   @Test
   @OperateOnDeployment("node1")
   @InSequence(5)
   public void stopCacheOnNode1() {
      System.out.println("Shutting down cachemanager on node1.");
      dfc1.removeListener(listenerNode1);
      ITestUtils.shutDownCache(dfc1, cacheName);
   }


}