package com.jboss.datagrid.test.notifications.utils;

import org.infinispan.configuration.cache.CacheMode;
import org.infinispan.configuration.cache.Configuration;
import org.infinispan.configuration.cache.ConfigurationBuilder;
import org.infinispan.configuration.global.GlobalConfiguration;
import org.infinispan.configuration.global.GlobalConfigurationBuilder;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.remoting.transport.jgroups.JGroupsTransport;

import static org.junit.Assert.assertTrue;

public class ITestUtils {

    public interface Condition {
        public boolean isSatisfied() throws Exception;
    }

    public static void eventually(String message, Condition ec, long timeout) {
        eventually(message, ec, timeout, 10);
    }

   /**
    *
    * @param message the identifying message for the {@link AssertionError}
    * @param ec when met then finish otherwise sleep next round
    * @param timeout after timeout being reached throw {@link AssertionError}
    * @param loops number of loops we use to sleep
    */
    public static void eventually(String message, Condition ec, long timeout, int loops) {
        if (loops <= 0) {
            throw new IllegalArgumentException("Number of loops must be positive");
        }
        long sleepDuration = timeout / loops;
        if (sleepDuration == 0) {
            sleepDuration = 1;
        }
        try {
            for (int i = 0; i < loops; i++) {

                if (ec.isSatisfied())
                    return;
                Thread.sleep(sleepDuration);
            }
            assertTrue(message, ec.isSatisfied());
        } catch (Exception e) {
            throw new RuntimeException("Unexpected!", e);
        }
    }

   /**
    *
    * @param numSecs sleep defined number of seconds
    */
    public static void sleepForSecs(double numSecs) {
        // give the elements time to be evicted
        try {
            Thread.sleep((long) (numSecs * 1000));
        } catch (InterruptedException e) {
        }
    }


   /**
    * Creates JDG configuration for cluster. The configuration is either invalidation or distribution dependent on
    * parameters passed.
    *
    * @param cacheName      the name of the cache to be created.
    * @param isInvalidation checks whether the cache should be invalidation or no.
    * @return the cache manager object.
    * @throws Exception
    */
   public static DefaultCacheManager configureCacheManagerForCluster(final String cacheName, final boolean isInvalidation) throws Exception {
      GlobalConfiguration glob = new GlobalConfigurationBuilder()
            .globalJmxStatistics().jmxDomain("AsyncNotifExecutorCache").allowDuplicateDomains(true)
            .transport().addProperty("configurationFile", "jgroups-tcp.xml").
                  distributedSyncTimeout(150000).
                  transport(new JGroupsTransport()).
                  asyncListenerExecutor().addProperty("maxThreads", "5").
                  build();
      ConfigurationBuilder c = new ConfigurationBuilder();

      if (isInvalidation) {
         c.clustering().cacheMode(CacheMode.INVALIDATION_SYNC);
      } else {
         c.clustering().cacheMode(CacheMode.DIST_ASYNC).hash().numOwners(2).transaction().locking().lockAcquisitionTimeout(60000);
      }

      Configuration cnf = c.build();
      DefaultCacheManager manager = new DefaultCacheManager(glob);
      manager.defineConfiguration(cacheName, cnf);
      return manager;
   }

   /**
    * Shutdowns the cachemanager, and the cache according to the passed parameters.
    *
    * @param cacheManager the cachemanager to shut down.
    * @param cacheName    the name of the cache which should be shut down.
    */
   public static void shutDownCache(final DefaultCacheManager cacheManager, final String cacheName) {
      cacheManager.getCache(cacheName).stop();
      cacheManager.stop();
   }
}
